@@ -10,8 +10,9 @@ import argcomplete
 
 from . import CHANGE_SET_FORMAT, __version__
 from . import stack_set
-from .aws import AWS
-from .helper import collect_vars
+from . import aws
+import boto3
+from .helper import collect_vars, with_artifacts
 
 STACK_HEADERS = ["Name", "Created At", "Updated At", "Status"]
 RESOURCE_HEADERS = ["Logical ID", "Physical ID", "Type", "Status"]
@@ -55,6 +56,7 @@ CONFIG_FILE_ARGUMENTS = {
     "use_previous_template": bool,
     "use_previous_parameters": bool,
     "s3": bool,
+    "artifacts": list,
 }
 
 
@@ -96,6 +98,7 @@ def main(cli_args):
     add_config_file_argument(template_parser)
     add_stack_variables_argument(template_parser)
     template_parser.add_argument("-y", "--yaml", help="print output as yaml", action="store_true")
+    add_artifacts_argument(template_parser)
     add_organization_account_template_variables(template_parser)
     template_parser.set_defaults(func=template)
 
@@ -116,6 +119,7 @@ def main(cli_args):
     add_config_file_argument(new_parser)
     add_stack_variables_argument(new_parser)
     add_s3_upload_argument(new_parser)
+    add_artifacts_argument(new_parser)
     add_resource_types(new_parser)
     add_organization_account_template_variables(new_parser)
     new_parser.set_defaults(func=new)
@@ -131,6 +135,7 @@ def main(cli_args):
     add_config_file_argument(change_parser)
     add_stack_variables_argument(change_parser)
     add_s3_upload_argument(change_parser)
+    add_artifacts_argument(change_parser)
     add_resource_types(change_parser)
     add_create_missing_argument(change_parser)
     add_organization_account_template_variables(change_parser)
@@ -140,6 +145,7 @@ def main(cli_args):
     # Deploy Command Arguments
     deploy_parser = subparsers.add_parser("deploy", description="Deploy the latest change set for a stack")
     add_aws_arguments(deploy_parser)
+    add_artifacts_argument(deploy_parser)
     add_stack_argument(deploy_parser)
     add_config_file_argument(deploy_parser)
     add_timeout_parameter(deploy_parser)
@@ -175,6 +181,7 @@ def main(cli_args):
     add_stack_parameters_argument(diff_parser)
     add_stack_tags_argument(diff_parser)
     add_organization_account_template_variables(diff_parser)
+    add_artifacts_argument(diff_parser)
     diff_parser.set_defaults(func=diff)
 
     # Resources Command Arguments
@@ -211,7 +218,7 @@ def main(cli_args):
 
     try:
         # Initialise the AWS Profile and Region
-        AWS.initialize(args_dict.get("region"), args_dict.get("profile"))
+        aws.initialize(args_dict.get("region"), args_dict.get("profile"))
 
         convert_role_name_to_arn(args)
 
@@ -235,7 +242,7 @@ def main(cli_args):
 
 def convert_role_name_to_arn(args):
     args_dict = vars(args)
-    sts = AWS.current_session().client("sts")
+    sts = boto3.client("sts")
     if args_dict.get("role_name") and not args_dict.get("role_arn"):
         account_id = sts.get_caller_identity()["Account"]
         args.role_arn = "arn:aws:iam::{}:role/{}".format(account_id, args.role_name)
@@ -346,8 +353,8 @@ def requires_stack(function):
 
 
 def add_aws_arguments(parser):
-    parser.add_argument("--region", help="The AWS region to use", metavar="REGION")
-    parser.add_argument("--profile", help="The AWS profile to use", metavar="PROFILE")
+    parser.add_argument("--region", help="The AWS region to use", metavar="REGION", default=None)
+    parser.add_argument("--profile", help="The AWS profile to use", metavar="PROFILE", default=None)
 
 
 def add_stack_argument(parser):
@@ -488,6 +495,15 @@ def add_s3_upload_argument(parser):
     parser.add_argument("--s3", help="Upload template to S3 before deployment", action="store_true")
 
 
+def add_artifacts_argument(parser):
+    parser.add_argument(
+        "--artifacts",
+        help="Add one or more artifacts to push to S3 before deployment",
+        nargs="+",
+        default=[],
+    )
+
+
 def add_resource_types(parser):
     parser.add_argument("--resource-types", help="Add Resource Types to the ChangeSet", action="store_true")
 
@@ -511,7 +527,9 @@ def template(args):
     from .loader import Loader
     import yaml
 
-    loader = Loader(variables=collect_vars(args))
+    variables = collect_vars(args)
+
+    loader = Loader(variables=variables)
     loader.load()
     if args.yaml:
         logger.info(
@@ -526,7 +544,7 @@ def template(args):
 def stacks(args):
     from texttable import Texttable
 
-    client = AWS.current_session().client("cloudformation")
+    client = boto3.client("cloudformation")
     stacks = client.describe_stacks()
     table = Texttable(max_width=150)
     table.add_rows([STACK_HEADERS])
@@ -550,8 +568,7 @@ def diff(args):
 def describe(args):
     from .change_set import ChangeSet
 
-    client = cloudformation_client()
-    change_set = ChangeSet(stack=args.stack, client=client)
+    change_set = ChangeSet(stack=args.stack)
     change_set.describe()
 
 
@@ -586,7 +603,7 @@ def change(args):
 
     client = cloudformation_client()
 
-    change_set = ChangeSet(stack=args.stack, client=client)
+    change_set = ChangeSet(stack=args.stack)
 
     change_set_type = "UPDATE"
     if args.create_missing:
@@ -624,7 +641,7 @@ def change(args):
 
 
 def cloudformation_client():
-    client = AWS.current_session().client("cloudformation")
+    client = boto3.client("cloudformation")
     return client
 
 
@@ -639,12 +656,13 @@ def wait_for_stack(function):
         options = {}
         if vars(args).get("timeout"):
             options["timeout"] = args.timeout
-        StackWaiter(stack_id, client, **options).wait(last_event)
+        StackWaiter(stack_id, **options).wait(last_event)
 
     return stack_wait_handler
 
 
 @requires_stack
+@with_artifacts
 @wait_for_stack
 def deploy(args, client):
     logger.info("Deploying Stack to {}".format(args.stack))
@@ -690,11 +708,10 @@ def new(args):
     from .change_set import ChangeSet
     from .loader import Loader
 
-    client = cloudformation_client()
     loader = Loader(variables=collect_vars(args))
     loader.load()
     logger.info("Creating change set for new stack, ...")
-    change_set = ChangeSet(stack=args.stack, client=client)
+    change_set = ChangeSet(stack=args.stack)
     change_set.create(
         template=loader.template(indent=None),
         change_set_type="CREATE",
